package org.esupportail.cas.server.util.crypt;

import jcrypt.jcrypt;
import md5.MD5;
import md5.MD5Crypt;

/**
 * This class offers encryption methods used by handlers encrypting
 * users' passwords and comparing them to a reference (file or
 * database).
 *
 * @author Jean-Baptiste Daniel <danielj at sourceforge.net>
 */
public abstract class Crypt {
	
	/**
	 * An array containing the encryptions supported by the module.
	 */
	private static String[] supportedEncryptions = {
			"plain", 
			"des", 
			"md5",
			"pammd5"};
	
	/**
	 * Tell if an encryption is supported.
	 * @param encryption an encryption
	 * 
	 * @return true if the encryption is supported, false otherwise.
	 */
	public static boolean isEncryptionSupported(final String encryption) {
		if (encryption.equals("plain")) {
			return true;
		}
		if (encryption.equals("des")) {
			return true;
		}
		if (encryption.equals("md5")) {
			return true;
		}
		if (encryption.equals("pammd5")) {
			return true;
		}
		return false;
	}

	/**
	 * Compare a password with its encryption using an encryption method.
	 * 
	 * @param encryption an encryption method (must be supported)
	 * @param password the password
	 * @param encryptedPassword the encrypted password
	 * 
	 * @return true if password matches, false otherwise
	 * @throws Exception when encryption is not supprted.
	 */
	public static boolean match(
			final String encryption,
			final String password,
			final String encryptedPassword) throws Exception {
		if (encryption.equals("plain")) {
			return encryptedPassword.equals(password);
		}
		if (encryption.equals("des")) {
			return matchDES(password, encryptedPassword);
		}
		if (encryption.equals("md5")) {
			return matchMD5(password, encryptedPassword);
		}
		if (encryption.equals("pammd5")) {
			return matchPAMMD5(password, encryptedPassword);
		}
		throw new Exception("Unsupported encryption method.");
	}
	
	/**
	 * Retrieve the salt of an MD5 encrypted string.
	 *
	 * @param encryptedString an MD5 encrypted string
	 *
	 * @return the salt of the encrypted string, or null on error.
	 */
	private static String getMD5Salt(final String encryptedString) {
		
		try {
			return encryptedString.split("\\$")[2];
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * MD5 Encryption of a string using a salt.
	 *
	 * @param password the password to encrypt
	 * @param salt the salt to encrypt with
	 *
	 * @return the encrypted string (null if the salt is null).
	 */    
	private static String cryptMD5(
			final String password, 
			final String salt) {
		if (salt == null) {
			return null;
		}
		return MD5Crypt.crypt(password, salt);
	}

	/**
	 * MD5 comparison.
	 *
	 * This method compares a clear password with a MD5 hash as created
	 * by 'md5sum' or MySQL's 'MD5()'.
	 *
	 * @param password a clear password.
	 * @param encryptedPassword an already encrypted password.
	 *
	 * @return true if password matches, false otherwise.
	 */
	private static boolean matchMD5(final String password,
			final String encryptedPassword) {
		return encryptedPassword.equals(MD5.md5crypt(password));
	}

	/**
	 * Linux PAM MD5 comparison.
	 *
	 * This method compares a clear password with a Linux PAM / OpenBSD
	 * / FreeBSD compatible md5-encoded password hash.
	 *
	 * @param password a clear password.
	 * @param encryptedPassword an already encrypted password.
	 *
	 * @return true if password matches, false otherwise.
	 */
	private static boolean matchPAMMD5(final String password,
			final String encryptedPassword) {
		return encryptedPassword.equals(cryptMD5(password, getMD5Salt(encryptedPassword)));
	}

	/**
	 * Retrieve the salt of a DES encrypted string.
	 *
	 * @param encryptedString a DES encrypted string
	 *
	 * @return the salt of the encrypted string, or null on error.
	 */
	private static String getDESSalt(final String encryptedString) {
		
		try {
			return encryptedString.substring(0, 2);
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * DES Encryption of a string using a salt.
	 *
	 * @param password the password to encrypt
	 * @param salt the salt to encrypt with
	 *
	 * @return the encrypted string (null if the salt is null).
	 */    
	private static String cryptDES(
			final String password, 
			final String salt) {
		if (salt == null) {
			return null;
		}
		return jcrypt.crypt(salt, password);
	}
	
	/**
	 * DES comparison.
	 *
	 * @param password a clear password.
	 * @param encryptedPassword an already encrypted password.
	 *
	 * @return true if password matches, false otherwise.
	 */
	private static boolean matchDES(final String password,
			final String encryptedPassword) {
		return encryptedPassword.equals(cryptDES(password, 
				getDESSalt(encryptedPassword)));
	}
	
}


